/*
    QJpegRest - An interactive JPEG decoder, with restoration algorithms
    Copyright (C) 2007  Lluís Batlle i Rossell

    Please find the license in the provided LICENSE file.
*/
#include <cstring>
#include "JPEGData.h"
#include "CoefsPlane.h"

/* From jutils.c in libjpeg */
const int CoefsPlane::jpeg_natural_order[DCTSize2] =
{
    0,  1,  8, 16,  9,  2,  3, 10,
    17, 24, 32, 25, 18, 11,  4,  5,
    12, 19, 26, 33, 40, 48, 41, 34,
    27, 20, 13,  6,  7, 14, 21, 28,
    35, 42, 49, 56, 57, 50, 43, 36,
    29, 22, 15, 23, 30, 37, 44, 51,
    58, 59, 52, 45, 38, 31, 39, 46,
    53, 60, 61, 54, 47, 55, 62, 63,
} ;

const int CoefsPlane::jpeg_zigzag_order[DCTSize2] = {
    0,  1,  5,  6, 14, 15, 27, 28,
    2,  4,  7, 13, 16, 26, 29, 42,
    3,  8, 12, 17, 25, 30, 41, 43,
    9, 11, 18, 24, 31, 40, 44, 53,
    10, 19, 23, 32, 39, 45, 52, 54,
    20, 22, 33, 38, 46, 51, 55, 60,
    21, 34, 37, 47, 50, 56, 59, 61,
    35, 36, 48, 49, 57, 58, 62, 63
};


CoefsPlane::CoefsPlane()
{
    ready = false;
}

CoefsPlane::CoefsPlane(float *_ptr, const unsigned int _width_in_blocks,
        const unsigned int _height_in_blocks)
{
    ptr = _ptr;
    width_in_blocks = _width_in_blocks;
    height_in_blocks = _height_in_blocks;
    ready = true;
}

CoefsPlane::CoefsPlane(const unsigned int _width_in_blocks,
        const unsigned int _height_in_blocks)
{
    allocate(_width_in_blocks, _height_in_blocks);

    ready = true;
}

void CoefsPlane::allocate(const unsigned int _width_in_blocks,
        const unsigned int _height_in_blocks)
{
    width_in_blocks = _width_in_blocks;
    height_in_blocks = _height_in_blocks;
    ptr = new float[width_in_blocks*height_in_blocks*64];
}

void CoefsPlane::free()
{
    delete[] ptr;
}

CoefsPlane CoefsPlane::newcopy()
{
    CoefsPlane n;

    n.allocate(width_in_blocks, height_in_blocks);
    n.setParameters(parameters);
    std::memcpy(n.getPtr(), ptr,
            sizeof(float)*width_in_blocks*height_in_blocks);

    return n;
}
