#include <gsl/gsl_matrix.h>
#include <stdio.h>
#include "libmat.h"

/* libmat. This library provides a gsl_matrix File Interchange Format.

The format consists in a header (the size written as text,
and then the gsl_matrix_fwrite data comes).

The size consists of two integers, rows and columns, separated by space and
ending in '\n'.

This way:
<rows> <columns>
<gsl_matrix_data>EOF

*/

int mat_fread(FILE *input, gsl_matrix **m)
{
	int numbers, result;
	int size1,size2;

	numbers = fscanf(input, "%i %i", &size1, &size2);
	if (numbers != 2)
	{
		fprintf(stderr,"The matrix couldn't be read.\n");
		return -1;
	}

	/* The error of fgetc will be catched on gsl_read_matrix */
	fgetc(input); /* We read the '\n' after the numbers */
	*m = gsl_matrix_alloc(size1, size2);
	if (*m == NULL)
	{
		fprintf(stderr,"The matrix couldn't be read. "
			"Memory allocation failed.\n");
		return -1;
	}


	result = gsl_matrix_fread(input, *m);
	if (result != 0)
	{
		fprintf(stderr,"The matrix couldn't be read.\n");
		gsl_matrix_free(*m);
		return -1;
	}

	return 0;
}

int mat_file_read(const char *fname, gsl_matrix **m)
{
	int result;
	FILE *input;

	input = fopen(fname, "r");
	if (input == NULL)
	{
		fprintf(stderr,"The matrix couldn't be read.\n");
		return -1;
	}

	result = mat_fread(input, m);

	fclose(input);

	return result;
}

int mat_fwrite(FILE *output, gsl_matrix *m)
{
	int result;

	result = fprintf(output, "%i %i\n", m->size1, m->size2);
	if (result < 0)
	{
		fprintf(stderr,"The matrix couldn't be written.\n");
		return -1;
	}

	result = gsl_matrix_fwrite(output, m);
	if (result != 0)
	{
		fprintf(stderr,"The matrix couldn't be written.\n");
		return -1;
	}
	return 0;
}

int mat_write(const char * fname, gsl_matrix *m)
{
	int result;
	FILE *output;

	output = fopen(fname, "w");
	if (output == NULL)
	{
		fprintf(stderr,"The matrix couldn't be written.\n");
		return -1;
	}

	result = mat_fwrite(output, m);

	fclose(output);

	return result;
}
